//I pledge on my honour and on the GITA that I have not given or received any unauthorized assistance on this assignment or any previous homework.

//References : Text: Computer Graphics Using OpenGL by Francis Hill. Prentice Hall.

#include<math.h>
#include <GL/gl.h>
#include <GL/glut.h>

#include<iostream>

using namespace std;

int screenWidth = 480;     //Height and width of the screen window
int screenHeight = 480;
float windowWidth = 3.0;   //Height and width of world window
float windowHeight = 3.0;
const float center_wW = windowWidth / 2.0;   //Center of world window
const float center_wH = windowHeight / 2.0;
GLdouble AR = windowWidth / windowHeight;    //Aspect ratio of world window
const int noOfSides = 5;                     //No. of sides of the polygon

static int state = 0;          //STATIC GLOBAL VARIABLE FOR MAINTAINING THE STATE

double phiiSquare = (((1 + sqrtf(5)) / 2) * ((1 + sqrtf(5)) / 2));    //Square of the golden ratio "phii"

class Point2D {                      
  public:
    GLfloat x, y;
};                          //Point class

Point2D CP;

/*****************Function for setting the world window *******************/

void setWindow(float left, float right, float bottom, float top)       
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(left, right, bottom, top);
}

/***************** Function for setting the viewport*******************/

void setViewport(int left, int right, int bottom, int top)
{
    glViewport(left, bottom, right - left, top - bottom);
}

/***************** Initialization of the environment*******************/

void myInit(void)
{
    glClearColor(1.0, 1.0, 1.0, 0.0);
    glColor3f(1.0f, 0.0f, 1.0f);
    glPointSize(2.0);
    glLineWidth(2.0);
    setWindow(0.0, windowWidth, 0.0, windowHeight);
    setViewport(0, screenWidth, 0, screenHeight);
    state = 1;
}

/*****************Function to move to a specific point *******************/

void moveTo(float x, float y)
{
    CP.x = x;
    CP.y = y;
}

/*****************Function to draw a line till a specific point  *******************/

void lineTo(float x, float y)
{
    glBegin(GL_LINES);
    glVertex2f(CP.x, CP.y);
    glVertex2f(x, y);
    glEnd();
    glFlush();

    CP.x = x;
    CP.y = y;
}

/***************** Function to draw a polygon *******************/

void ngon(int n, float cx, float cy, float radius, float rotAngle,
	  Point2D pts[])
{
    if (n < 3)
	return;
    double angle = rotAngle * 3.14159265 / 180;         //to rotate to desired degree
    double angleInc = 2 * 3.14159265 / n;
    moveTo(radius * cos(angle) + cx, radius * sin(angle) + cy);
    pts[0].x = radius * cos(angle) + cx;
    pts[0].y = radius * sin(angle) + cy;                 
    for (int k = 1; k <= n; k++) {           
	angle += angleInc;
	pts[k].x = radius * cos(angle) + cx;
	pts[k].y = radius * sin(angle) + cy;
	lineTo(radius * cos(angle) + cx, radius * sin(angle) + cy);
    }    //draws the polygon

}

/***************** Function to draw a rosette*******************/

float Rosette(int N, float radius, float rotAngle)
{
    float dist;
    Point2D pt[N];               //array containing points of the polygon
    ngon(N, windowWidth / 2.0, windowHeight / 2.0, radius, rotAngle, pt);  //call to draw a polygon
    for (int i = 0; i < N - 1; i++) {
	for (int j = i + 1; j < N; j++) {
	    moveTo(pt[i].x, pt[i].y);
	    lineTo(pt[j].x, pt[j].y);
	}
    }

    dist = sqrtf(abs((pt[1].x - pt[0].x) * (pt[1].x - pt[0].x) + ((pt[1].y - pt[0].y) * (pt[1].y - pt[0].y))));
    return dist;     //returns side length of the polygon for further calculations
}

/***************** Function to control the display *******************/

void myDisplay(void)
{
    glClear(GL_COLOR_BUFFER_BIT);
    glColor3f(1.0f, 0.0f, 1.0f);
    int j = 1;
    float distance;
    float alpha;		//SIDE LENGTH
    float angR;
    float Radius = 1.0;
    float rotAngle = 90;
    distance = Rosette(noOfSides, Radius, rotAngle);
    if (state > 1) {
	for (int i = 2; i <= state; i++) {
	    j = i % 7;
//Draw the polygon with the required color
	    switch (j) {
	    case 0:
		glColor3f(0.0f, 1.0f, 1.0f);
		break;
	    case 1:
		glColor3f(1.0f, 0.0f, 1.0f);
		break;
	    case 2:
		glColor3f(1.0f, 1.0f, 0.0f);
		break;
	    case 3:
		glColor3f(0.0f, 0.0f, 0.0f);
		break;
	    case 4:
		glColor3f(1.0f, 0.0f, 0.0f);
		break;
	    case 5:
		glColor3f(0.0f, 1.0f, 0.0f);
		break;
	    case 6:
		glColor3f(0.0f, 0.0f, 1.0f);
		break;
	    default:
		break;
	    }
	    alpha = distance / phiiSquare;
	    angR = (360 / (2 * noOfSides)) * 3.14159265 / 180;
	    Radius = alpha / (sin(angR) * 2);  //calculate radius of new internal polygon
	    if (i % 2 == 0)
		rotAngle = -90;              
	    else
		rotAngle = 90;
	    moveTo(windowWidth / 2.0, windowHeight / 2.0);
	    distance = Rosette(noOfSides, Radius, rotAngle);
	}
    }

    glutSwapBuffers();
    glFlush();
}

/***************** Function to control by the keyboard *******************/

void myKeyboard(unsigned char Key, int mouseposX, int mouseposY)
{
    switch (Key) {
//Zoom in
    case 'z':
	glClear(GL_COLOR_BUFFER_BIT);
	windowWidth *= 0.9;
	windowHeight = windowWidth / AR;
	setWindow(0.0, windowWidth, 0.0, windowHeight);
        cout<<"Zoomed in \n";
	break;
    case 'Z':
	glClear(GL_COLOR_BUFFER_BIT);
	windowWidth *= 0.9;
	windowHeight = windowWidth / AR;
	setWindow(0.0, windowWidth, 0.0, windowHeight);
        cout<<"Zoomed in \n";
	break;
//Return to the original figure
    case 'q':
        state = 1;
        windowWidth = 3.0;
        windowHeight = 3.0;
        setWindow(0.0, windowWidth, 0.0, windowHeight);
        cout<<"Back to original figure \n";
        break;
    case 'Q':
	state = 1;
	windowWidth = 3.0;
	windowHeight = 3.0;
	setWindow(0.0, windowWidth, 0.0, windowHeight);
        cout<<"Back to original figure \n";
        break;
//Zoom out
    case 'x':
	glClear(GL_COLOR_BUFFER_BIT);
	windowWidth = windowWidth / 0.9;
	windowHeight = windowWidth / AR;
	setWindow(0.0, windowWidth, 0.0, windowHeight);
        cout<<"Zoomed out \n";
        break;
//Zoom out 
    case 'X':
	glClear(GL_COLOR_BUFFER_BIT);                      
	windowWidth = windowWidth / 0.9;
	windowHeight = windowWidth / AR;
	setWindow(0.0, windowWidth, 0.0, windowHeight);
        cout<<"Zoomed out \n";
        break;
    default:
	break;
    }

    glutPostRedisplay();  //calls myDisplay()
}

/***************** Function to control by the mouse *******************/

void myMouse(int button, int status, int x, int y)
{
    if (button == GLUT_LEFT_BUTTON && status == GLUT_DOWN) {
	cout << "Left Button Down Detected \n";
	++state;              //Increments the state variable to draw one more polygon
	glutPostRedisplay();  //calls myDisplay() 
    }

}

/***************** Function to control the shape of the screen window*******************/

//Maintains the SAME ASPECT RATIO even if the screen is resized

void myReshape(GLsizei W, GLsizei H)
{
    int WiS, WiE, HeS, HeE;

    if (AR > W / H) {
	HeS = (int) (H / 2 - W / (2 * AR));
	HeE = (int) (H / 2 + W / (2 * AR));
	setViewport(0, W, HeS, HeE);
    } else {
	WiS = (int) (W / 2 - (H * AR) / 2);
	WiE = (int) (W / 2 + (H * AR) / 2);
	setViewport(WiS, WiE, 0, H);
    }

    glutPostRedisplay();
}

/***************** The main program *******************/

int main(int argc, char **argv)
{
    glutInit(&argc, argv);	// initialize the toolkit
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);	// set display mode
    glutInitWindowSize(screenWidth, screenHeight);	// set window size
    glutInitWindowPosition(100, 150);	// set window position on screen
    glutCreateWindow("my first attempt");	// open the screen window
    glutDisplayFunc(myDisplay);	// register redraw function
    glutReshapeFunc(myReshape);
    glutKeyboardFunc(myKeyboard);
    glutMouseFunc(myMouse);
    myInit();
    glutMainLoop();		// go into a perpetual loop
    return 0;
}
